@@ -5,11 +5,12 @@
 所有中间件类型消费者的抽象基类。使实现不同中间件的消费者尽可能代码少。
 整个流程最难的都在这里面。因为要实现多种并发模型，和对函数施加20运行种控制方式，所以代码非常长。
 """
-
+import typing
 import abc
 import copy
 # from multiprocessing import Process
 import datetime
+# noinspection PyUnresolvedReferences
 import pytz
 import json
 import logging
@@ -202,9 +203,10 @@ class ConsumersManager:
     def show_all_consumer_info(cls):
         # nb_print(f'当前解释器内，所有消费者的信息是：\n  {cls.consumers_queue__info_map}')
         # if only_print_on_main_process(f'当前解释器内，所有消费者的信息是：\n  {json.dumps(cls.consumers_queue__info_map, indent=4, ensure_ascii=False)}'):
-        for _, consumer_info in cls.consumers_queue__info_map.items():
-            stdout_write(f'{time.strftime("%H:%M:%S")} "{consumer_info["where_to_instantiate"]}" '
-                         f' \033[0;30;44m{consumer_info["queue_name"]} 的消费者\033[0m\n')
+        if not cls._has_show_conusmers_info:
+            for _, consumer_info in cls.consumers_queue__info_map.items():
+                stdout_write(f'{time.strftime("%H:%M:%S")} "{consumer_info["where_to_instantiate"]}" '
+                             f' \033[0;30;44m{consumer_info["queue_name"]} 的消费者\033[0m\n')
         cls._has_show_conusmers_info = True
 
     @staticmethod
@@ -386,7 +388,6 @@ class AbstractConsumer(LoggerLevelSetterMixin, metaclass=abc.ABCMeta, ):
         self._concurrent_mode_dispatcher = ConcurrentModeDispatcher(self)
         if self._concurrent_mode == 4:
             self._run = self._async_run  # 这里做了自动转化，使用async_run代替run
-        self.__check_monkey_patch()
 
         self._logger_prefix = logger_prefix
         self._log_level = log_level
@@ -417,6 +418,7 @@ class AbstractConsumer(LoggerLevelSetterMixin, metaclass=abc.ABCMeta, ):
         self._lock_for_count_execute_task_times_every_unit_time = Lock()
         self._current_time_for_execute_task_times_every_unit_time = time.time()
         self._consuming_function_cost_time_total_every_unit_time = 0
+        self._last_execute_task_time = time.time()  # 最近一次执行任务的时间。
 
         self._msg_num_in_broker = 0
         self._last_timestamp_when_has_task_in_queue = 0
@@ -505,6 +507,12 @@ class AbstractConsumer(LoggerLevelSetterMixin, metaclass=abc.ABCMeta, ):
     # noinspection PyAttributeOutsideInit
     def start_consuming_message(self):
         ConsumersManager.show_all_consumer_info()
+        try:
+            self._concurrent_mode_dispatcher.check_all_concurrent_mode()
+            self.__check_monkey_patch()
+        except Exception:
+            traceback.print_exc()
+            os._exit(4444)  # noqa
         self.logger.warning(f'开始消费 {self._queue_name} 中的消息')
         if self._is_send_consumer_hearbeat_to_redis:
             self._distributed_consumer_statistics = DistributedConsumerStatistics(self._queue_name, self.consumer_identification)
@@ -530,6 +538,7 @@ class AbstractConsumer(LoggerLevelSetterMixin, metaclass=abc.ABCMeta, ):
     def _print_message_get_from_broker(self, broker_name, msg):
         if isinstance(msg, (dict, list)):
             msg = json.dumps(msg, ensure_ascii=False)
+        # print(999)
         if self._is_show_message_get_from_broker:
             self.logger.debug(f'从 {broker_name} 中间件 的 {self._queue_name} 中取出的消息是 {msg}')
 
@@ -552,13 +561,6 @@ class AbstractConsumer(LoggerLevelSetterMixin, metaclass=abc.ABCMeta, ):
         return concurrent_info
 
     def _run(self, kw: dict, ):
-        function_only_params = _delete_keys_and_return_new_dict(kw['body'], )
-        if self.__get_priority_conf(kw, 'do_task_filtering') and self._redis_filter.check_value_exists(
-                function_only_params):  # 对函数的参数进行检查，过滤已经执行过并且成功的任务。
-            self.logger.warning(f'redis的 [{self._redis_filter_key_name}] 键 中 过滤任务 {kw["body"]}')
-            self._confirm_consume(kw)
-            return
-
         t_start_run_fun = time.time()
         self._run_consuming_function_with_confirm_and_retry(kw, current_retry_times=0,
                                                             function_result_status=FunctionResultStatus(
@@ -568,11 +570,17 @@ class AbstractConsumer(LoggerLevelSetterMixin, metaclass=abc.ABCMeta, ):
         with self._lock_for_count_execute_task_times_every_unit_time:
             self._execute_task_times_every_unit_time += 1
             self._consuming_function_cost_time_total_every_unit_time += time.time() - t_start_run_fun
+            self._last_execute_task_time = time.time()
             if time.time() - self._current_time_for_execute_task_times_every_unit_time > self._unit_time_for_count:
+                avarage_function_spend_time = round(self._consuming_function_cost_time_total_every_unit_time / self._execute_task_times_every_unit_time, 4)
                 msg = f'{self._unit_time_for_count} 秒内执行了 {self._execute_task_times_every_unit_time} 次函数 [ {self.consuming_function.__name__} ] ,' \
-                      f'函数平均运行耗时 {round(self._consuming_function_cost_time_total_every_unit_time / self._execute_task_times_every_unit_time, 4)} 秒'
+                      f'函数平均运行耗时 {avarage_function_spend_time} 秒'
                 if self._msg_num_in_broker != -1:
-                    msg += f''' ，预计还需要 {time_util.seconds_to_hour_minute_second(self._msg_num_in_broker / self._execute_task_times_every_unit_time * self._unit_time_for_count)} 时间 才能执行完成 {self._msg_num_in_broker}个剩余的任务'''
+                    if hasattr(self, '_distributed_consumer_statistics'):
+                        active_consumer_num = self._distributed_consumer_statistics.active_consumer_num
+                    else:
+                        active_consumer_num = 1
+                    msg += f''' ，预计还需要 {time_util.seconds_to_hour_minute_second(self._msg_num_in_broker * avarage_function_spend_time / active_consumer_num)} 时间 才能执行完成 {self._msg_num_in_broker}个剩余的任务'''
                 self.logger.info(msg)
                 self._current_time_for_execute_task_times_every_unit_time = time.time()
                 self._consuming_function_cost_time_total_every_unit_time = 0
@@ -638,19 +646,8 @@ class AbstractConsumer(LoggerLevelSetterMixin, metaclass=abc.ABCMeta, ):
         self._result_persistence_helper.save_function_result_to_mongo(function_result_status)
 
     async def _async_run(self, kw: dict, ):
-        """虽然和上面有点大面积重复相似，这个是为了asyncio模式的，asyncio模式真的和普通同步模式的代码思维和形式区别太大，
-        框架实现兼容async的消费函数很麻烦复杂，连并发池都要单独写"""
-        function_only_params = _delete_keys_and_return_new_dict(kw['body'], )
-        # if self.__get_priority_conf(kw, 'do_task_filtering') and self._redis_filter.check_value_exists(
-        #         function_only_params):  # 对函数的参数进行检查，过滤已经执行过并且成功的任务。
-        if self.__get_priority_conf(kw, 'do_task_filtering'):
-            is_exists = await simple_run_in_executor(self._redis_filter.check_value_exists,
-                                                     function_only_params)
-            if is_exists:
-                self.logger.warning(f'redis的 [{self._redis_filter_key_name}] 键 中 过滤任务 {kw["body"]}')
-                # self._confirm_consume(kw)
-                await simple_run_in_executor(self._confirm_consume, kw)
-                return
+        # """虽然和上面有点大面积重复相似，这个是为了asyncio模式的，asyncio模式真的和普通同步模式的代码思维和形式区别太大，
+        # 框架实现兼容async的消费函数很麻烦复杂，连并发池都要单独写"""
         t_start_run_fun = time.time()
         await self._async_run_consuming_function_with_confirm_and_retry(kw, current_retry_times=0,
                                                                         function_result_status=FunctionResultStatus(
@@ -659,11 +656,17 @@ class AbstractConsumer(LoggerLevelSetterMixin, metaclass=abc.ABCMeta, ):
         # 异步调度不存在线程并发，不需要加锁。
         self._execute_task_times_every_unit_time += 1
         self._consuming_function_cost_time_total_every_unit_time += time.time() - t_start_run_fun
+        self._last_execute_task_time = time.time()
         if time.time() - self._current_time_for_execute_task_times_every_unit_time > self._unit_time_for_count:
-            msg = f'''{self._unit_time_for_count} 秒内执行了 {self._execute_task_times_every_unit_time} 次函数 [ {self.consuming_function.__name__} ] ,''' + \
-                  f'''函数平均运行耗时 {round(self._consuming_function_cost_time_total_every_unit_time / self._execute_task_times_every_unit_time, 4)} 秒'''
+            avarage_function_spend_time = round(self._consuming_function_cost_time_total_every_unit_time / self._execute_task_times_every_unit_time, 4)
+            msg = f'{self._unit_time_for_count} 秒内执行了 {self._execute_task_times_every_unit_time} 次函数 [ {self.consuming_function.__name__} ] ,' \
+                  f'函数平均运行耗时 {avarage_function_spend_time} 秒'
             if self._msg_num_in_broker != -1:
-                msg += f''' ，预计还需要 {time_util.seconds_to_hour_minute_second(self._msg_num_in_broker / self._execute_task_times_every_unit_time * self._unit_time_for_count)} 时间 才能执行完成 {self._msg_num_in_broker}个剩余的任务'''
+                if hasattr(self, '_distributed_consumer_statistics'):
+                    active_consumer_num = self._distributed_consumer_statistics.active_consumer_num
+                else:
+                    active_consumer_num = 1
+                msg += f''' ，预计还需要 {time_util.seconds_to_hour_minute_second(self._msg_num_in_broker * avarage_function_spend_time / active_consumer_num)} 时间 才能执行完成 {self._msg_num_in_broker}个剩余的任务'''
             self.logger.info(msg)
             self._current_time_for_execute_task_times_every_unit_time = time.time()
             self._consuming_function_cost_time_total_every_unit_time = 0
@@ -788,6 +791,12 @@ class AbstractConsumer(LoggerLevelSetterMixin, metaclass=abc.ABCMeta, ):
             self._requeue(kw)
             time.sleep(self.time_interval_for_check_do_not_run_time)
             return
+        function_only_params = _delete_keys_and_return_new_dict(kw['body'], )
+        if self.__get_priority_conf(kw, 'do_task_filtering') and self._redis_filter.check_value_exists(
+                function_only_params):  # 对函数的参数进行检查，过滤已经执行过并且成功的任务。
+            self.logger.warning(f'redis的 [{self._redis_filter_key_name}] 键 中 过滤任务 {kw["body"]}')
+            self._confirm_consume(kw)
+            return
         publish_time = _get_publish_time(kw['body'])
         msg_expire_senconds_priority = self.__get_priority_conf(kw, 'msg_expire_senconds')
         if msg_expire_senconds_priority and time.time() - msg_expire_senconds_priority > publish_time:
@@ -855,23 +864,54 @@ class AbstractConsumer(LoggerLevelSetterMixin, metaclass=abc.ABCMeta, ):
                 f'现在时间是 {time_util.DatetimeConverter()} ，现在时间是在 {self._do_not_run_by_specify_time} 之间，不运行')
             return True
 
+    def wait_for_possible_has_finish_all_tasks(self, minutes: int = 3):
+        """
+        判断队列所有任务是否消费完成了。
+        由于是异步消费，和存在队列一边被消费，一边在推送，或者还有结尾少量任务还在确认消费者实际还没彻底运行完成。  但有时候需要判断 所有任务，务是否完成，提供一个不精确的判断，要搞清楚原因和场景后再慎用。
+        一般是和celery一样，是永久运行的后台任务，永远无限死循环去任务执行任务，但有的人有判断是否执行完成的需求。
+        :param minutes: 消费者连续多少分钟没执行任务任务 并且 消息队列中间件中没有，就判断为消费完成。
+        :return:
+
+        """
+        if minutes <= 1:
+            raise ValueError('疑似完成任务，判断时间最少需要设置为3分钟内,最好是是10分钟')
+        no_task_time = 0
+        while 1:
+            # noinspection PyBroadException
+            message_count = self._msg_num_in_broker
+            # print(message_count,self._last_execute_task_time,time.time() - self._last_execute_task_time)
+            if message_count == 0 and self._last_execute_task_time != 0 and (time.time() - self._last_execute_task_time) > minutes * 60:
+                no_task_time += 30
+            else:
+                no_task_time = 0
+            time.sleep(30)
+            if no_task_time > minutes * 60:
+                break
+
     def __str__(self):
         return f'队列为 {self.queue_name} 函数为 {self.consuming_function} 的消费者'
 
 
+def wait_for_possible_has_finish_all_tasks_by_conusmer_list(consumer_list: typing.List[AbstractConsumer], minutes: int = 3):
+    """
+   判断多个消费者是否消费完成了。
+   由于是异步消费，和存在队列一边被消费，一边在推送，或者还有结尾少量任务还在确认消费者实际还没彻底运行完成。  但有时候需要判断 所有任务，务是否完成，提供一个不精确的判断，要搞清楚原因和场景后再慎用。
+   一般是和celery一样，是永久运行的后台任务，永远无限死循环去任务执行任务，但有的人有判断是否执行完成的需求。
+   :param minutes: 消费者连续多少分钟没执行任务任务 并且 消息队列中间件中没有，就判断为消费完成。
+   :return:
+
+    """
+    with BoundedThreadPoolExecutor(len(consumer_list)) as pool:
+        for consumer in consumer_list:
+            pool.submit(consumer.wait_for_possible_has_finish_all_tasks(minutes))
+
+
 # noinspection PyProtectedMember
 class ConcurrentModeDispatcher(LoggerMixin):
 
     def __init__(self, consumerx: AbstractConsumer):
         self.consumer = consumerx
-        if ConsumersManager.global_concurrent_mode is not None and self.consumer._concurrent_mode != ConsumersManager.global_concurrent_mode:
-            ConsumersManager.show_all_consumer_info()
-            # print({self.consumer._concurrent_mode, ConsumersManager.global_concurrent_mode})
-            if not {self.consumer._concurrent_mode, ConsumersManager.global_concurrent_mode}.issubset({1, 4, 5}):
-                # threding、asyncio、solo 这几种模式可以共存。但同一个解释器不能同时选择 gevent + 其它并发模式，也不能 eventlet + 其它并发模式。
-                raise ValueError('由于猴子补丁的原因，同一解释器中不可以设置两种并发类型,请查看显示的所有消费者的信息，'
-                                 '搜索 concurrent_mode 关键字，确保当前解释器内的所有消费者的并发模式只有一种')
-        self._concurrent_mode = ConsumersManager.global_concurrent_mode = self.consumer._concurrent_mode
+        self._concurrent_mode = self.consumer._concurrent_mode
         self.timeout_deco = None
         if self._concurrent_mode in (1, 5):
             self.timeout_deco = decorators.timeout
@@ -882,6 +922,19 @@ class ConcurrentModeDispatcher(LoggerMixin):
         self.logger.warning(f'{self.consumer} 设置并发模式'
                             f'为{ConsumersManager.get_concurrent_name_by_concurrent_mode(self._concurrent_mode)}')
 
+    def check_all_concurrent_mode(self):
+        if ConsumersManager.global_concurrent_mode is not None and self.consumer._concurrent_mode != ConsumersManager.global_concurrent_mode:
+            ConsumersManager.show_all_consumer_info()
+            # print({self.consumer._concurrent_mode, ConsumersManager.global_concurrent_mode})
+            if not {self.consumer._concurrent_mode, ConsumersManager.global_concurrent_mode}.issubset({1, 4, 5}):
+                # threding、asyncio、solo 这几种模式可以共存。但同一个解释器不能同时选择 gevent + 其它并发模式，也不能 eventlet + 其它并发模式。
+                raise ValueError('由于猴子补丁的原因，同一解释器中不可以设置两种并发类型,请查看显示的所有消费者的信息，'
+                                 '搜索 concurrent_mode 关键字，确保当前解释器内的所有消费者的并发模式只有一种(或可以共存),'
+                                 'asyncio threading single_thread 并发模式可以共存，但gevent和threading不可以共存，'
+                                 'gevent和eventlet不可以共存')
+
+        ConsumersManager.global_concurrent_mode = self.consumer._concurrent_mode
+
     def build_pool(self):
         if self.consumer._concurrent_pool is not None:
             return self.consumer._concurrent_pool
@@ -902,6 +955,7 @@ class ConcurrentModeDispatcher(LoggerMixin):
             self.consumer._concurrent_pool = self.consumer._specify_concurrent_pool if self.consumer._specify_concurrent_pool is not None else pool_type(
                 self.consumer._concurrent_num, loop=self.consumer._specify_async_loop)
         else:
+            # print(pool_type)
             self.consumer._concurrent_pool = self.consumer._specify_concurrent_pool if self.consumer._specify_concurrent_pool is not None else pool_type(
                 self.consumer._concurrent_num)
         # print(self._concurrent_mode,self.consumer._concurrent_pool)
@@ -930,40 +984,6 @@ class ConcurrentModeDispatcher(LoggerMixin):
             # elif self._concurrent_mode ==5:
 
 
-# noinspection DuplicatedCode
-def wait_for_possible_has_finish_all_tasks(queue_name: str, minutes: int, send_stop_to_broker=0,
-                                           broker_kind: int = 0, ):
-    """
-      由于是异步消费，和存在队列一边被消费，一边在推送，或者还有结尾少量任务还在确认消费者实际还没彻底运行完成。  但有时候需要判断 所有任务，务是否完成，提供一个不精确的判断，要搞清楚原因和场景后再慎用。
-    :param queue_name: 队列名字
-    :param minutes: 连续多少分钟没任务就判断为消费已完成
-    :param send_stop_to_broker :发送停止标志到中间件，这回导致消费退出循环调度。
-    :param broker_kind: 中间件种类
-    :return:
-    """
-    if minutes <= 1:
-        raise ValueError('疑似完成任务，判断时间最少需要设置为2分钟内,最好是是10分钟')
-    pb = get_publisher(queue_name, broker_kind=broker_kind)
-    no_task_time = 0
-    while 1:
-        # noinspection PyBroadException
-        try:
-            message_count = pb.get_message_count()
-        except Exception as e:
-            nb_print(e)
-            message_count = -1
-        if message_count == 0:
-            no_task_time += 30
-        else:
-            no_task_time = 0
-        time.sleep(30)
-        if no_task_time > minutes * 60:
-            break
-    if send_stop_to_broker:
-        pb.publish({'stop': 1})
-    pb.close()
-
-
 class DistributedConsumerStatistics(RedisMixin, LoggerMixinDefaultWithFileHandler):
     """
     为了兼容模拟mq的中间件（例如redis，他没有实现amqp协议，redis的list结构和真mq差远了），获取一个队列有几个连接活跃消费者数量。
